<?php

// index.php :: Primary program script, evil alien overlord, you decide.

require_once '../src/lib.php';
require_once '../src/router.php';

if (!file_exists('../.installed')) redirect('install.php');

$controlrow = get_control_row();

// Login (or verify) if not logged in.
if (($userrow = checkcookies()) === false) {
    if (isset($_GET['do']) && $_GET['do'] === 'verify') {
        header("Location: users.php?do=verify");
		exit;
    }

    header("Location: login.php?do=login");
	exit;
}

// Close game.
if ((bool) $controlrow["gameopen"] === false) {
	display("The game is currently closed for maintanence. Please check back later.", "Game Closed");
	exit;
}

// Force verify if the user isn't verified yet.
if ((bool) $controlrow["verifyemail"] && (bool) $userrow["verify"] === false) {
	header("Location: users.php?do=verify");
	exit;
}

// Block user if he/she has been banned.
if ($userrow["authlevel"] === 2) {
	exit("Your account has been blocked.");
}

require_once '../src/towns.php';
require_once '../src/explore.php';
require_once '../src/fight.php';
require_once '../src/heal.php';

$r = new Router;

$r->get('/', function() {
	global $userrow;

    if ($userrow["currentaction"] == "In Town") {
        $page = dotown();
        $title = "In Town";
    } elseif ($userrow["currentaction"] == "Exploring") {
        $page = doexplore();
        $title = "Exploring";
    } elseif ($userrow["currentaction"] == "Fighting")  {
        redirect('/fight');
    }

    display($page, $title);
});

$r->get('/ninja', function() {
	exit('NINJA! 🥷');
});

$r->get('/inn', 'inn');
$r->post('/inn', 'inn');
$r->get('/buy', 'buy');
$r->get('/buy2/:id', 'buy2');
$r->post('/buy3/:id', 'buy3');
// $r->get('/sell', 'sell');
$r->get('/maps', 'maps');
$r->get('/maps2/:id', 'maps2');
$r->post('/maps3/:id', 'maps3');
$r->get('/gotown/:id', 'travelto');

$r->post('/move', 'move');

$r->get('/fight', 'fight');
$r->post('/fight', 'fight');
$r->get('/victory', 'victory');
$r->get('/drop', 'drop');
$r->post('/drop', 'drop');
$r->get('/dead', 'dead');

$r->get('/verify', function() {
	redirect('users.php?do=verify');
});
$r->get('/spell/:id', 'healspells');
$r->get('/showchar', 'showchar');
$r->get('/onlinechar/:id', 'onlinechar');
$r->get('/showmap', 'showmap');
$r->get('/babblebox', 'babblebox');
$r->post('/babblebox', 'babblebox');

// [code, handler, params, middleware]
$l = $r->lookup($_SERVER['REQUEST_METHOD'], $_SERVER['REQUEST_URI']);

if ($l['code'] !== 200) exit($l['code']);
if (!empty($l['middleware'])) foreach ($l['middleware'] as $middleware) $middleware();
$l['handler'](...$l['params'] ?? []);

function donothing()
{
    global $userrow;

    if ($userrow["currentaction"] == "In Town") {
        $page = dotown();
        $title = "In Town";
    } elseif ($userrow["currentaction"] == "Exploring") {
        $page = doexplore();
        $title = "Exploring";
    } elseif ($userrow["currentaction"] == "Fighting")  {
        redirect('/fight');
    }

    display($page, $title);
}

/**
 * Spit out the main town page.
 */
function dotown()
{
    global $userrow, $controlrow;

	$townrow = get_town_by_xy($userrow['longitude'], $userrow['latitude']);
	if ($townrow === false) display("There is an error with your user account, or with the town data. Please try again.","Error");

	$townrow["news"] = "";
	$townrow["whosonline"] = "";
	$townrow["babblebox"] = "";

    // News box. Grab latest news entry and display it. Something a little more graceful coming soon maybe.
    if ($controlrow["shownews"] == 1) {
        $newsrow = db()->query('SELECT * FROM news ORDER BY id DESC LIMIT 1;')->fetchArray(SQLITE3_ASSOC);
        $townrow["news"] = '<div class="title">Latest News</div>';
        $townrow["news"] .= "<span class=\"light\">[".prettydate($newsrow["postdate"])."]</span><br>".nl2br($newsrow["content"]);
    }

    // Who's Online. Currently just members. Guests maybe later.
    if ($controlrow["showonline"] == 1) {
		$onlinequery = db()->query("SELECT id, username FROM users WHERE strftime('%s', onlinetime) >= strftime('%s', 'now') - 600 ORDER BY username");

		$online_count = 0;
		$online_rows = [];

		while ($onlinerow = $onlinequery->fetchArray(SQLITE3_ASSOC)) {
			$online_count++;
			$online_rows[] = "<a href=\"/onlinechar/{$onlinerow["id"]}\">".$onlinerow["username"]."</a>";
		}

        $townrow["whosonline"] = '<div class="title">Who\'s Online</div>';
        $townrow["whosonline"] .= "There are <b>$online_count</b> user(s) online within the last 10 minutes: ";
        $townrow["whosonline"] .= rtrim(implode(', ', $online_rows), ', ');
    }

    if ($controlrow["showbabble"] == 1) {
        $townrow["babblebox"] = <<<HTML
			<div class="title">Babble Box</div>
			<iframe src="/babblebox" name="sbox" width="100%" height="250" frameborder="0" id="bbox">
				Your browser does not support inline frames! The Babble Box will not be available until you upgrade to
				a newer <a href="http://www.mozilla.org" target="_new">browser</a>.
			</iframe>
		HTML;
    }

    return parsetemplate(gettemplate("towns"), $townrow);
}

/**
 * Just spit out a blank exploring page. Exploring without a GET string is normally when they first log in, or when
 * they've just finished fighting.
 */
function doexplore()
{
	return <<<HTML
		<table width="100%">
			<tr><td class="title"><img src="/img/title_exploring.gif" alt="Exploring" /></td></tr>
			<tr><td>
				You are exploring the map, and nothing has happened. Continue exploring using the direction buttons or the Travel To menus.
			</td></tr>
		</table>
	HTML;
}

function showchar()
{
    global $userrow, $controlrow;

	$userrow["experience"] = number_format($userrow["experience"]);
	$userrow["gold"] = number_format($userrow["gold"]);

	$userrow["plusexp"] = $userrow["expbonus"] != 0
		? "<span class=\"light\">(" . ($userrow["expbonus"] > 0 ? "+" : "") . $userrow["expbonus"] . "%)</span>"
		: "";

	$userrow["plusgold"] = $userrow["goldbonus"] != 0
		? "<span class=\"light\">(" . ($userrow["goldbonus"] > 0 ? "+" : "") . $userrow["goldbonus"] . "%)</span>"
		: "";

	$levelrow = db()->query("SELECT `{$userrow["charclass"]}_exp` FROM levels WHERE id=? LIMIT 1;", [$userrow['level'] + 1])->fetchArray(SQLITE3_ASSOC);
	$userrow["nextlevel"] = $userrow['level'] < 99 ? number_format($levelrow[$userrow["charclass"]."_exp"]) : '<span class="light">None</span>';

	$userrow['charclass'] = match ((int) $userrow['charclass']) {
		1 => $controlrow["class1name"],
		2 => $controlrow["class2name"],
		3 => $controlrow["class3name"]
	};

	$spells = db()->query('SELECT id, name FROM spells;');
    $userspells = explode(',', $userrow['spells']);
    $userrow["magiclist"] = '';
    while ($spellrow = $spells->fetchArray(SQLITE3_ASSOC)) {
        $spell = false;
        foreach($userspells as $b) if ($b == $spellrow["id"]) $spell = true;
        if ($spell == true) $userrow["magiclist"] .= $spellrow["name"]."<br>";
    }
    if ($userrow["magiclist"] == "") $userrow["magiclist"] = "None";

    $array = ["content" => parsetemplate(gettemplate("showchar"), $userrow), "title" => "Character Information"];
    echo parsetemplate("<html>\n" . gettemplate("minimal"), $array);
}

function onlinechar($id)
{
    global $controlrow;

	$query = db()->query('SELECT * FROM users WHERE id=? LIMIT 1;', [$id]);
    if ($query !== false) { $userrow = $query->fetchArray(SQLITE3_ASSOC); } else { display("No such user.", "Error"); }
	unset($userrow['password']);

    $userrow["experience"] = number_format($userrow["experience"]);
	$userrow["gold"] = number_format($userrow["gold"]);

	$userrow["plusexp"] = $userrow["expbonus"] != 0
		? "<span class=\"light\">(" . ($userrow["expbonus"] > 0 ? "+" : "") . $userrow["expbonus"] . "%)</span>"
		: "";

	$userrow["plusgold"] = $userrow["goldbonus"] != 0
		? "<span class=\"light\">(" . ($userrow["goldbonus"] > 0 ? "+" : "") . $userrow["goldbonus"] . "%)</span>"
		: "";

	$levelrow = db()->query("SELECT `{$userrow["charclass"]}_exp` FROM levels WHERE id=? LIMIT 1;", [$userrow['level'] + 1])->fetchArray(SQLITE3_ASSOC);
	$userrow["nextlevel"] = $userrow['level'] < 99 ? number_format($levelrow[$userrow["charclass"]."_exp"]) : '<span class="light">None</span>';

	$userrow['charclass'] = match ((int) $userrow['charclass']) {
		1 => $controlrow["class1name"],
		2 => $controlrow["class2name"],
		3 => $controlrow["class3name"]
	};

    display(parsetemplate(gettemplate("onlinechar"), $userrow), "Character Information");
}

function showmap()
{
    $array = ["content" => "<center><img src=\"/img/map.gif\" alt=\"Map\" /></center>", "title" => "Map"];
    echo parsetemplate("<html>\n" . gettemplate("minimal"), $array);
}

function babblebox()
{
    global $userrow;

    if (isset($_POST["babble"])) {
        $safecontent = makesafe($_POST["babble"]);
        if (!empty($safecontent)) {
			db()->query('INSERT INTO babble (posttime, author, babble) VALUES (CURRENT_TIMESTAMP, ?, ?);', [$userrow['username'], $safecontent]);
		}
        redirect('/babblebox');
    }

    $babblebox['content'] = '';
	$query = db()->query('SELECT * FROM babble ORDER BY id DESC LIMIT 40;');
    while ($babblerow = $query->fetchArray(SQLITE3_ASSOC)) {
        $new = "<div class=\"message\">[<b>{$babblerow["author"]}</b>] {$babblerow["babble"]}</div>\n";
        $babblebox["content"] = $new . $babblebox["content"];
    }
    $babblebox["content"] .= '<form action="/babblebox" method="post" style="margin-top: 1rem;"><input type="text" name="babble" maxlength="255" style="width: 100%;"><br><input type="submit" name="submit" value="Babble"><input type="reset" name="reset" value="Clear"></form>';

    echo parsetemplate("<html>\n" . gettemplate("babblebox"), $babblebox);
}

/**
 * NINJA! 🥷
 */
function ninja(): void
{
    exit('NINJA! 🥷');
}
